package org.krasnow.cng.domain;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class StandardTreeNode implements TreeNode, Comparable {

    protected Object data;
    //protected List children;
    protected List children;
    protected StandardTreeNode parent;
    protected long level;

    protected int branchSize;
    protected int height;
    protected int totalHeight;
    protected float averageHeight;
    protected int degree;

    public StandardTreeNode(){
        data = null;
        parent = null;
        children = new ArrayList();
        branchSize = 1;
    }
    public StandardTreeNode(Object d){
        data = d;
        parent = null;
        children = new ArrayList();
        branchSize = 1;
    }
    public void setParent(StandardTreeNode p){
    	setParent(p, true);
    }
    public void setParent(StandardTreeNode p, boolean addChild){
        parent = p;
        if (p != null && addChild){
        	if (!p.getChildren().contains(this)){
        		p.addChildNode(this, false);
        		p.updateBranchSize();
        	}
        }
    }
    public void addChildNode(StandardTreeNode n){
    	addChildNode(n, true);
    }
    public void addChildNode(StandardTreeNode n, boolean setParent){
    	if (setParent){
    		n.setParent(this, false);
    	}
//    	childrenMap.put(n.getData(), n);
    	children.add(n);
    	StandardTreeNode.updateBranchStatisticsUp(this);
    }
    
    public void setData(Object d) throws Exception{
        data = d;
    }
    public Object getData(){
        return data;
    }
    
    public List getChildren(){
        return children;
    }
    public void setChildren(List children){
        this.children = children;
    }

    public String toString(){
        return ""+data;
    }
    
    public int getBranchSize(){
        return branchSize;
    }
    
    public int getBranchSizeRecursively(){
    	int size = 1;
    	
    	List list = children;
        for (int i = 0; i < list.size(); i++){
     	   size += ((StandardTreeNode)list.get(i)).getBranchSizeRecursively();
        }
    	return size;
    }

    public void updateBranchSize(){
       branchSize = 1;
       StandardTreeNode node;
       List list = children;
       for (int i = 0; i < list.size(); i++){
    	   node = (StandardTreeNode)list.get(i);
    	   branchSize += node.getBranchSize();
       }
       
       if (parent != null){
          parent.updateBranchSize();
       }
    }

    public boolean hasChildren(){
       if (children == null || children.size() == 0){
          return false;
       }
       else{
          return true;
       }
    }

    public void detach(){
       TreeNode p = parent;
       if (p != null){
	       p.getChildren().remove(this);
    	   StandardTreeNode.updateBranchStatisticsUp((StandardTreeNode)p);
	       parent = null;
       }
    }

    public String toStringLargeThenSmall(){
       String str = data.toString();
       List list = children;
       while (!list.isEmpty()){
    	   int largest = 0;
    	   int size = 0;
	       for (int i = 0; i < list.size(); i++){
	    	   if (((StandardTreeNode)list.get(i)).getBranchSize() > size){
	    		   size = ((StandardTreeNode)list.get(i)).getBranchSize();
	    		   largest = i;
	    	   }
	       }
	       str.concat(((StandardTreeNode)list.get(largest)).toStringLargeThenSmall());
	       list.remove(largest);
       }
       return str;
    }

    public String toStringSmallThenLarge(){
        String str = data.toString();
        List list = children;
        while (!list.isEmpty()){
     	   int smallest = 0;
     	   int size = 10000;
 	       for (int i = 0; i < list.size(); i++){
 	    	   if (((StandardTreeNode)list.get(i)).getBranchSize() < size){
 	    		   size = ((StandardTreeNode)list.get(i)).getBranchSize();
 	    		   smallest = i;
 	    	   }
 	       }
 	       str.concat(((StandardTreeNode)list.get(smallest)).toStringSmallThenLarge());
 	       list.remove(smallest);
        }
        return str;
    }

    public static void updateStatisticsDown(SwcTreeNode node){
    	StandardTreeNode current, child;
    	LinkedStack assembleStack = new LinkedStack();
    	LinkedStack branchStack = new LinkedStack();
    	int branchSize, maxHeight, totalHeight, degree;
    	assembleStack.push(node);
    	branchStack.push(node);
    	while (!assembleStack.isEmpty()){
    		current = (SwcTreeNode)assembleStack.pop();
    		for (Iterator it = current.getChildren().iterator(); it.hasNext();){
    			child = (SwcTreeNode)it.next();
    			assembleStack.push(child);
    			branchStack.push(child);
    		}
    	}
    	assembleStack = null;
    	while (!branchStack.isEmpty()){
    		current = (StandardTreeNode)branchStack.pop();
    		branchSize = 1;
    		degree = totalHeight = maxHeight = 0;
    		if (current.hasChildren()){
	    		for (Iterator it = current.getChildren().iterator(); it.hasNext();){
	    			child = (StandardTreeNode)it.next();
	
	    			branchSize += child.getBranchSize();
	    			degree += child.getDegree();
	    			maxHeight = child.getHeight() + 1;
	    			// totalLength is the total length of the children plus 1 for each child below (branchSize)
	    			totalHeight += child.getTotalHeight()
	    				+ child.getDegree();
	    		}
    		}
    		else{
    			degree++;
    		}
    		current.setBranchSize(branchSize);
    		current.setDegree(degree);
    		current.setHeight(maxHeight);
    		current.setTotalHeight(totalHeight);
    		// branchSize-1 so this node is not included
    		current.setAverageHeight((float)totalHeight/degree);
    	}
    	branchStack = null;
    }

    public static void updateBranchStatisticsUp(StandardTreeNode node){
    	StandardTreeNode current = node, child;
    	int branchSize, maxHeight, totalHeight, degree;
    	while (current != null){
    		branchSize = 1;
    		degree = totalHeight = maxHeight = 0;
    		if (current.hasChildren()){
	    		for (Iterator it = current.getChildren().iterator(); it.hasNext();){
	    			child = (StandardTreeNode)it.next();
	
	    			branchSize += child.getBranchSize();
	    			degree += child.getDegree();
	    			maxHeight = child.getHeight() + 1;
	    			// totalLength is the total length of the children plus 1 for each child below (branchSize)
	    			totalHeight += child.getTotalHeight()
	    				+ child.getDegree();
	    		}
    		}
    		else{
    			degree++;
    		}
    		current.setBranchSize(branchSize);
    		current.setDegree(degree);
    		current.setHeight(maxHeight);
    		current.setTotalHeight(totalHeight);
    		// branchSize-1 so this node is not included
    		current.setAverageHeight((float)totalHeight/degree);
    		
   			current = (StandardTreeNode)current.getParent();
    	}
    }

    public int compareTo(Object o){
    	if (o == null){
    		return 1;
    	}
    	StandardTreeNode node = (StandardTreeNode)o;
    	if (branchSize != node.getBranchSize()){
    		return branchSize - node.getBranchSize();	
    	}
    	else if (height != node.getHeight()){
    		return height - node.getHeight();
    	}
    	else if (averageHeight != node.getAverageHeight()){
    		return (Math.round(averageHeight - node.getAverageHeight()));
    	}
    	else {
    		return degree - node.getDegree();
    	}
    }
    public float getAverageHeight() {
		return averageHeight;
	}
	protected void setAverageHeight(float averageHeight) {
		this.averageHeight = averageHeight;
	}
	public int getHeight() {
		return height;
	}
	protected void setHeight(int height) {
		this.height = height;
	}
	public int getDegree() {
		return degree;
	}
	protected void setDegree(int degree) {
		this.degree = degree;
	}
	public long getLevel() {
		return level;
	}
	protected void setLevel(long level) {
		this.level = level;
	}
	public int getTotalHeight() {
		return totalHeight;
	}
	protected void setTotalHeight(int totalHeight) {
		this.totalHeight = totalHeight;
	}
	public StandardTreeNode getParent() {
		return parent;
	}
	public TreeNode getTreeNodeParent() {
		return parent;
	}
	public boolean hasParent(){
		return parent != null;
	}
	protected void setBranchSize(int branchSize) {
		this.branchSize = branchSize;
	}
	public boolean isLeaf(){
		return children == null || children.isEmpty();
	}
    
}
